
/**
 * Created by Jordan Blake
 */



 var express = require('express');

 var app = express();

 var server = require('http').Server(app);

 var WordApi = undefined; //WordApi

 const path = require('path');



var LOGGING = false;


 // Allow some files to be server over HTTP
 app.use(express.static(path.join( __dirname, '/')));

 var responseActions = {

   respond: function(resData, responseObject, resXml) {

     var o2x = require('object-to-xml');

     resData.sort(function(a, b) {

       return a.word.lemma > b.word.lemma;

     });

     resData = responseActions.dataMassage.titleByLemma(resData);

     if (!resXml) {

       responseObject.write(JSON.stringify(resData, null, 4));

       return responseObject.end();

     }

     responseObject.write(o2x({
       words: resData
     }));

     return responseObject.end();


   },

   dataMassage: {

     titleByLemma: function(wordArray) {

       var count = 0;

       var lastLemma = false;

      if(LOGGING)
       console.log("titleByLemma(wordArray): Array type:" + typeof(wordArray));

       if (wordArray instanceof Array && LOGGING) {
         console.log("Array type:array");
       }


       for (var x = 0; x < wordArray.length; x++) {


         if (lastLemma !== wordArray[x].word.lemma) {

           count = 0;

           lastLemma = wordArray[x].word.lemma

         }

         count += 1;

         wordArray[x].word.titleTag = wordArray[x].word.lemma + "(" + count + "):";

       }


       return wordArray;

     },

     containsObject: function(obj, list) {
       var i;
       for (i = 0; i < list.length; i++) {
         if (list[i] === obj) {
           return true;
         }
       }

       return false;
     },

     uniqueMembers: function(list) {

       var listOut = [];

       for (var x = 0; x < list.length; x++) {

         if (!responseActions.dataMassage.containsObject(list[x], listOut)) {
           listOut.push(list[x]);
         }
       }
       return listOut;
     }
   }
 };


 //easy reference to dataMassage
 var dataMassage = responseActions.dataMassage;

 const getDistinctSynsetIds = function(lexdomainname) {
   var sids = [];
 };

 const getWordResponse = function(words, synsetids, options, responseObject, resXml) {

   var usingSynsets = words.constructor !== Array && synsetids.constructor == Array ? true : false;

   var wordDataOut = [];

   if ((words.constructor !== Array && synsetids.constructor !== Array)) {

     responseObject.end();

     return false;
   }


   if (!usingSynsets) {

     if (words.length > 50) {

       //limit words to 50 total
       words = words.slice(0, 50);

     }

     words = dataMassage.uniqueMembers(words);
     wordDataOut = WordApi.getWordsByLemmas(words);
   } else {

     if (synsetids.length > 10) {

       var response = {
         status: "invalid-request",
         message: "no more than 10 synsetids per request"
       }

       responseObject.end(JSON.stringify(response, null, 4));
       return false;
     }

     synsetids = dataMassage.uniqueMembers(synsetids);
     wordDataOut = WordApi.getWordsBySynsetids(synsetids);

   }


   if (options.fullData || options.semlinks) {


     var semlinks = WordApi.db.getCollection('semlinksspecial');

     var semlinksPerRecursiveLimit = 10;

     for (var x = 0; x < wordDataOut.length; x++) {

       var sid = parseInt(wordDataOut[x].word.synsetid);

       var data = WordApi.getSemlinksBySynsetid(sid, options.recursive);

       wordDataOut[x].word.semlinks = [];

       for (var z = 0; z < data.length; z++) {

         wordDataOut[x].word.semlinks.push(data[z]);

       }
     }

   }


   if (options.fullData || options.samples) {

     for (var x = 0; x < wordDataOut.length; x++) {

       var word = wordDataOut[x].word;

       word.samples = WordApi.getSamplesBySynsetid(word.wordid, word.synsetid);

     }
   }

   if (options.fullData || options.statistics) {

     for (var x = 0; x < wordDataOut.length; x++) {

       var word = wordDataOut[x].word;

       word.statistics = WordApi.getStatisticsByWordidAndPos(word.wordid, word.pos);

     }
   }

   if(LOGGING)
   console.log(JSON.stringify(wordDataOut, null, 4));

   responseObject.write(JSON.stringify(wordDataOut, null, 4));

   responseObject.end();

 }

 var ServerFxns = {

   httpInit: function() {

     app.get('/', function(req, res) {

       res.sendFile(path.join( __dirname, 'examples/search-lexicon.html'));

     });

     app.get('/wn-license', function(req, res) {

       res.sendFile(path.join( __dirname, 'data/wordnet-3-license.txt'));

     });

     app.get('/dexicon/api/basic', function(req, res) {

       if(LOGGING)
       console.log("recieved basic request");

       var words = req.query && req.query.words ? req.query.words.split(',') : false,

         synsetids = req.query && req.query.synsetids ? req.query.synsetids.split(',') : false,

         resXml = req.query && req.query.restype && req.query.restype.toLowerCase() === 'xml' ? true : false;

       var options = {
         fullData: false,
         statistics: true,
         semlinks: true,
         samples: true,
         recursive: false
       };

       getWordResponse(words, synsetids, options, res, resXml);
       res.end('TODO : rig response');

     });


     app.get('/dexicon/api/samples?', function(req, res) {

       if(LOGGING)
       console.log("recieved request");

       var words = req.query && req.query.words ? req.query.words.split(',') : false,

         synsetids = req.query && req.query.synsetids ? req.query.synsetids.split(',') : false,


         resXml = req.query && req.query.restype && req.query.restype.toLowerCase() === 'xml' ? true : false;

       var options = {
         fullData: false,
         statistics: false,
         semlinks: false,
         samples: true,
         recursive: false
       };

       getWordResponse(words, synsetids, options, res, resXml);

     });


     app.get('/dexicon/api/semlinks?', function(req, res) {

       if(LOGGING)
       console.log("recieved request");

       var words = req.query && req.query.words ? req.query.words.split(',') : false,

         synsetids = req.query && req.query.synsetids ? req.query.synsetids.split(',') : false,


         resXml = req.query && req.query.restype && req.query.restype.toLowerCase() === 'xml' ? true : false;

       var options = {
         fullData: false,
         statistics: false,
         semlinks: true,
         samples: false,
         recursive: true
       };

       getWordResponse(words, synsetids, options, res, resXml);

     });


     app.get('/dexicon/api/statistics?', function(req, res) {

       if(LOGGING)
       console.log("recieved request");

       var words = req.query && req.query.words ? req.query.words.split(',') : false,

         synsetids = req.query && req.query.synsetids ? req.query.synsetids.split(',') : false,


         resXml = req.query && req.query.restype && req.query.restype.toLowerCase() === 'xml' ? true : false;

       var options = {
         fullData: false,
         statistics: true,
         semlinks: false,
         samples: false,
         recursive: false
       };

       getWordResponse(words, synsetids, options, res, resXml);

     });


     app.get('/dexicon/api/deep?', function(req, res) {

        if(LOGGING)
       console.log("recieved deep request");

       var words = req.query && req.query.words ? req.query.words.split(',') : false,

         synsetids = req.query && req.query.synsetids ? req.query.synsetids.split(',') : false,


         resXml = req.query && req.query.restype && req.query.restype.toLowerCase() === 'xml' ? true : false;

       var options = {
         fullData: true,
         statistics: true,
         semlinks: true,
         samples: true,
         recursive: true
       };

       getWordResponse(words, synsetids, options, res, resXml);

     });
   }
 };










const lexiconServer = {

   start: function(wordApi, port) {

     port = port || 3000;

     WordApi = wordApi;

     ServerFxns.httpInit();

     server.listen(port);

     console.log('LexiconServer is listening at port:' + port);

   }
 };


 var wordnetApi = require('./data/api.js');

 const port = 3000;

 wordnetApi.start(function(){
   console.log('wordnetApi initialized...');
   lexiconServer.start(wordnetApi, port);
 });

 module.exports = {
   wordnetApi:wordnetApi,
   server:lexiconServer
 };
